package com.buglabs.dragonfly.cdt.poky.ui;

import java.io.IOException;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import com.buglabs.dragonfly.cdt.poky.ui.utils.QEMUUtils;

/**
 * The activator class controls the plug-in life cycle
 */
public class PokyActivator extends AbstractUIPlugin {
	
	public static final String PLUGIN_ID = "com.buglabs.dragonfly.cdt.poky.ui";
	public static final String QEMU_HOST_NAME = "192.168.7.200";
	public static final String QEMU_USER_NAME = "root";
	public static final String QEMU_HOST_DIR = "/usr";

	// The shared instance
	private static PokyActivator plugin;
	private MessageConsoleStream stream;
	private String qemu_password = null;
	
	// images
	public static final String IMAGE_POKY_WIZARD_ICON = "images/poky_new_wizard.gif"; //$NON-NLS-1$
	
	/**
	 * The constructor
	 */
	public PokyActivator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		
		MessageConsole console = new MessageConsole("Poky Console", null);
		console.activate();
		ConsolePlugin.getDefault().getConsoleManager().addConsoles(new IConsole[]{ console });
		stream = console.newMessageStream();
	}
	
	public void writeToConsole(String message) throws IOException{
		stream.write(message);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}
	
	public void logError(String message, Throwable e){
		getLog().log(new Status(Status.ERROR, PLUGIN_ID, 0, message , e));
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static PokyActivator getDefault() {
		return plugin;
	}
	
	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 * 
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	protected void initializeImageRegistry(ImageRegistry imageRegistry) {
		imageRegistry.put(IMAGE_POKY_WIZARD_ICON, getImageDescriptor(IMAGE_POKY_WIZARD_ICON).createImage());
	}

	public void setQEMUPassword(String password) {
		qemu_password = password;
	}
	
	public String getQEMUPassword(){
		if(qemu_password == null){
			QEMUUtils.promptPassword();
		}
		return qemu_password;
	}

}
