package com.buglabs.dragonfly.cdt.poky.ui.launch;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

import com.buglabs.dragonfly.cdt.poky.ui.PokyActivator;

/**
 * Tab for Poky launch configuration
 * @author akravets
 *
 */
public class PokyTab extends AbstractLaunchConfigurationTab {

	private Text compilerRoot;
	private Text compilerPokyRoot;
	private String compiler;

	private Text kernelLocation;
	private Text rootfsLocation;

	private GridData rbData;
	private GridData labelData;
	private GridData gridData;
	private GridData textData;
	private GridData gdButton;
	private Button rootBrowseBtn;
	private Button rootRadioBtn;
	private Button pokyRootRadioBtn;
	private Button pokyRootBrowseBtn;

	private static final String RADIO_BUTTON_TYPE = "RADIO_BUTTON_TYPE";
	private static final String RADIO_ROOT = "RADIO_ROOT";
	private static final String RADIO_POKY_ROOT = "RADIO_POKY_ROOT";

	public PokyTab() {
		refresh();
	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout(1, false));

		initGridData();

		Label compilerDescription = new Label(composite, SWT.NONE);
		compilerDescription
				.setText("Define toolchian that will be used in compilation process");

		Group compilerGroup = new Group(composite, SWT.NONE);
		compilerGroup.setText("Cross Compiler");
		compilerGroup.setLayout(new GridLayout(3, false));
		compilerGroup.setLayoutData(gridData);

		createCompilerOptionsSection(compilerGroup);

		Label targetDescription = new Label(composite, SWT.NONE);
		targetDescription.setText("Define kernel and root file system");

		Group targetGroup = new Group(composite, SWT.NONE);
		targetGroup.setText("Target Options");
		targetGroup.setLayout(new GridLayout(3, false));
		targetGroup.setLayoutData(gridData);

		createTargetOptionsSection(targetGroup);

		setControl(composite);
	}

	private void initGridData() {
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.grabExcessHorizontalSpace = true;

		rbData = new GridData();
		rbData.widthHint = 300;
		rbData.horizontalSpan = 3;

		labelData = new GridData();
		labelData.horizontalIndent = 20;

		textData = new GridData(GridData.FILL_HORIZONTAL);
		textData.grabExcessHorizontalSpace = true;

		gdButton = new GridData();
		gdButton.horizontalAlignment = SWT.BEGINNING;
	}

	/*
	 * Creates compiler options group
	 */
	private void createCompilerOptionsSection(final Group compilerGroup) {

		Listener radioGroupListener = new Listener() {
			public void handleEvent(Event event) {
				if (event.widget instanceof Button) {
					Button b = (Button) event.widget;
					String data = (String) b.getData();
					if (data.equals(RADIO_ROOT)) {
						pokyRootBrowseBtn.setEnabled(false);
						compilerPokyRoot.setEnabled(false);

						rootBrowseBtn.setEnabled(true);
						compilerRoot.setEnabled(true);
					} else if (data.equals(RADIO_POKY_ROOT)) {
						rootBrowseBtn.setEnabled(false);
						compilerRoot.setEnabled(false);

						pokyRootBrowseBtn.setEnabled(true);
						compilerPokyRoot.setEnabled(true);
					}
				}
			}
		};

		rootRadioBtn = new Button(compilerGroup, SWT.RADIO);
		rootRadioBtn.setData(RADIO_ROOT);

		rootRadioBtn.setSelection(true);
		rootRadioBtn.setText("Use external toolchain");
		rootRadioBtn.setLayoutData(rbData);
		rootRadioBtn.addListener(SWT.Selection, radioGroupListener);

		Label rootLabel = new Label(compilerGroup, SWT.NONE);
		rootLabel.setText("Root ");
		rootLabel.setLayoutData(labelData);

		compilerRoot = new Text(compilerGroup, SWT.BORDER);
		compilerRoot.setLayoutData(textData);

		rootBrowseBtn = new Button(compilerGroup, SWT.PUSH);
		rootBrowseBtn.setText("Browse...");
		rootBrowseBtn.setLayoutData(gdButton);

		rootBrowseBtn.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseUp(MouseEvent e) {
				DirectoryDialog fd = new DirectoryDialog(((Button) e.widget)
						.getShell());
				compilerRoot.setText(fd.open());
				compiler = compilerRoot.getText();
				refresh();
			}

		});

		pokyRootRadioBtn = new Button(compilerGroup, SWT.RADIO);
		pokyRootRadioBtn.setSelection(false);
		pokyRootRadioBtn.setText("Use a full Poky tree");
		pokyRootRadioBtn.setLayoutData(rbData);
		pokyRootRadioBtn.setData(RADIO_POKY_ROOT);
		pokyRootRadioBtn.addListener(SWT.Selection, radioGroupListener);

		Label pokyRootLabel = new Label(compilerGroup, SWT.NONE);
		pokyRootLabel.setText("Poky Root ");
		pokyRootLabel.setLayoutData(labelData);

		compilerPokyRoot = new Text(compilerGroup, SWT.BORDER);
		compilerPokyRoot.setLayoutData(textData);

		pokyRootBrowseBtn = new Button(compilerGroup, SWT.PUSH);
		pokyRootBrowseBtn.setText("Browse...");
		pokyRootBrowseBtn.setLayoutData(gdButton);
		pokyRootBrowseBtn.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseUp(MouseEvent e) {
				DirectoryDialog fd = new DirectoryDialog(((Button) e.widget)
						.getShell());
				compilerPokyRoot.setText(fd.open());
				compiler = compilerPokyRoot.getText();
				refresh();
			}

		});

	}

	/*
	 * Creates target options group
	 */
	private void createTargetOptionsSection(Group targetGroup) {
		Label kernelLabel = new Label(targetGroup, SWT.NONE);
		kernelLabel.setText("Kernel");
		kernelLabel.setLayoutData(labelData);

		kernelLocation = new Text(targetGroup, SWT.BORDER);
		kernelLocation.setLayoutData(textData);
		kernelLocation.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				refresh();
			}
		});

		Button kernelBtn = new Button(targetGroup, SWT.PUSH);
		kernelBtn.setText("Browse...");
		kernelBtn.setLayoutData(gdButton);

		kernelBtn.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseUp(MouseEvent e) {
				FileDialog fd = new FileDialog(((Button) e.widget).getShell());
				kernelLocation.setText(fd.open());
				refresh();
			}

		});

		Label rootfsLabel = new Label(targetGroup, SWT.NONE);
		rootfsLabel.setText("Root file system");
		rootfsLabel.setLayoutData(labelData);

		rootfsLocation = new Text(targetGroup, SWT.BORDER);
		rootfsLocation.setLayoutData(textData);

		Button rootfBtn = new Button(targetGroup, SWT.PUSH);
		rootfBtn.setText("Browse...");
		rootfBtn.setLayoutData(gdButton);

		rootfBtn.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseUp(MouseEvent e) {
				FileDialog fd = new FileDialog(((Button) e.widget).getShell());
				rootfsLocation.setText(fd.open());
				refresh();
			}

		});
	}

	public String getName() {
		return "Poky";
	}

	public void initializeFrom(ILaunchConfiguration configuration) {
		pokyRootBrowseBtn.setEnabled(false);
		compilerPokyRoot.setEnabled(false);
		
		try {
			Map props = configuration.getAttribute(PokyLaunchConfigurationDelegate.PROPERTIES, new HashMap());
			String kernel = (String) props.get(PokyLaunchConfigurationDelegate.KERNEL);
			String rootfs = (String) props.get(PokyLaunchConfigurationDelegate.ROOTFS);
			
			kernelLocation.setText(kernel);
			rootfsLocation.setText(rootfs);
		} catch (CoreException e) {
			PokyActivator.getDefault().getLog().log(new Status(Status.ERROR, PokyActivator.PLUGIN_ID, 0, "Unable to initialize configuration", e));
		}
	}

	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		configuration.setAttribute(PokyLaunchConfigurationDelegate.COMPILER, compiler);
		configuration.setAttribute(PokyLaunchConfigurationDelegate.KERNEL, kernelLocation.getText());
		configuration.setAttribute(PokyLaunchConfigurationDelegate.ROOTFS, rootfsLocation.getText());
	}

	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
		try {
			PokyLaunchConfigurationInitializer.initializeSystemProperties(configuration);
		} catch (CoreException e1) {
			PokyActivator.getDefault().getLog().log(new Status(Status.ERROR, PokyActivator.PLUGIN_ID, 0, "Unable to set default configuration", e1));
		}
	}

	public boolean isValid(ILaunchConfiguration launchConfig) {
		return true;
	}

	private void refresh() {
		setDirty(true);
		updateLaunchConfigurationDialog();
	}
}
