package com.buglabs.dragonfly.cdt.poky.ui.launch;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.TimerTask;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import com.buglabs.dragonfly.cdt.poky.ui.PokyActivator;
import com.buglabs.dragonfly.cdt.poky.ui.utils.UIUtils;

public class ShellHelper {
	private boolean debug = true;

	private String command;

	private int editCode;

	private Exception exception;

	private Process process;

	private PrintStream stdin;

	private PokyActivator pokyActivator;
	
	public ShellHelper() {
		pokyActivator = PokyActivator.getDefault();
	}
	/**
	 * Executes a given command line with root rights and default time out of five seconds.
	 * 
	 * @see #execute(String, String,int)
	 * @param cmd the command line to execute
	 * @param input the input to feed to the command via 'stdin', can be null
	 * @return the combined output of 'stdout' and 'stderr' from the command
	 * @throws Exception
	 */
	public String sudoExecute(String cmd, final String input, String password) throws Exception {
		return sudoExecute(cmd,input,password,5000);
	}
	/**
	 * Executes a given command line with root rights.
	 * 
	 * @see #execute(String, String,int)
	 * @param cmd the command line to execute
	 * @param input the input to feed to the command via 'stdin', can be null
	 * @return the combined output of 'stdout' and 'stderr' from the command
	 * @param timeout in milliseconds after which the process is aborted, use zero for no timeout
	 * @throws Exception
	 */
	public String sudoExecute(String cmd, final String input, String password, int timeout) throws Exception {
		return execute("sudo -S " + cmd, password+input!=null?input:"",5000);
	}
	/**
	 * Executes a given command line with default time out of five seconds.
	 * 
	 * @see #execute(String, String,int)
	 * @param cmd the command line to execute
	 * @param input the input to feed to the command via 'stdin', can be null
	 * @return the combined output of 'stdout' and 'stderr' from the command
	 */
	public String execute(String cmd, final String input) {
		return execute(cmd, input, 5000);
	}

	/**
	 * Executes a given command line, aborting after given timeout.
	 * 
	 * After calling this method you should check for error by calling the getExitCode(). A non zero
	 * positive value indicates an error, typically Unix/Linux processes can only use values in the
	 * range 0..255 inclusive.
	 * If an exception was thrown during the call execute method
	 * the {@link #getExitCode()} returns -1 and you can get the exception with the {@link #getException()} call
	 * 
	 * @see #getExitCode
	 * @see #getCommand
	 * @see #getException
	 * @param cmd the command line to execute
	 * @param input the input to feed to the command via 'stdin', can be null
	 * @param timeout in milliseconds after which the process is aborted, use zero for no timeout
	 * @return the combined output of 'stdout' and 'stderr' from the command
	 */
	public String execute(String cmd, final String input, int timeout) {
		exception=null;
		OutputStream output = new ByteArrayOutputStream();
		command = cmd;
		editCode = -1;
		try {
			if (debug)
				pokyActivator.writeToConsole(cmd);
			process = Runtime.getRuntime().exec(cmd);
			stdin = new PrintStream(process.getOutputStream());
			InputStream stdout = process.getInputStream();
			createForkedPipe(stdout, debug ? System.out : null, output);
			InputStream errout = process.getErrorStream();
			createForkedPipe(errout, debug ? System.err : null, output);

			/*if (input != null) {
				stdin.println(input);
				stdin.close();
			}*/

			java.util.Timer timer = new java.util.Timer();
			if (timeout > 0)
				timer.schedule(new TimerTask() {
					public void run() {
						process.destroy();
					}
				}, timeout);
			int res = process.waitFor();
			timer.cancel();
			editCode = res;
		} catch (IOException e) {
			exception=e;
			System.err.println(e.getMessage());
		} catch (Exception e) {
			exception=e;
			e.printStackTrace();
		}
		return output.toString();
	}
	/**
	 * Gets the exit code from the last command executed, should be checked after each 
	 * invokation of {@code execute} to verify success. Typically 0 value indicates success,
	 * no Unix/Linux script/program should not be able to return a code outside 0..255,
	 * a return code of -1 indicates an exception in attempting to execute the command line
	 * in other words an exception was thrown before or during execution of the command
	 * and it was not possible to capture the actual exit code from the command.
	 * 
	 * @return the exit code
	 */

	public int getExitCode() {
		return editCode;
	}
	/**
	 * Gets the last command that has been executed (succesfully or not in other word the command line that 
	 * returned the exit code you can query with getExitCode()
	 * @return the command line
	 */
	public String getCommand() {
		return command;
	}
	/**
	 * Gets the exception that was thrown during, before or after the execution of the command line within
	 * the execute method, if any. If no exception was thrown, returns null.
	 * @return the exception or null
	 */
	public Exception getException() {
		return exception;
	}

	private void createForkedPipe(final InputStream in, final OutputStream out1, final OutputStream out2) {
		Thread thread = new Thread() {
			private String line;

			public void run() {
				try{
					BufferedReader input = new BufferedReader(new InputStreamReader(in));
					while ((line = input.readLine()) != null) {
						System.out.println("line: " + line);
						if(line.indexOf("password for") != -1 || line.indexOf("only root") != -1){
							PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable(){
								public void run() {
									IWorkbenchWindow win = PlatformUI.getWorkbench().getActiveWorkbenchWindow();

									Shell s = new Shell();

									if (win != null) {
										s = win.getShell();
									} else {
										s = new Shell();
									}

									PasswordDialog d = new PasswordDialog(s, line);
									String password = "";
									int userSelection = d.open();
									if(userSelection == Dialog.OK){
										password = d.getPwd();

										try {
											Process process = Runtime.getRuntime().exec(command);
											PrintStream stdin = new PrintStream(process.getOutputStream());

											if (password != null) {
												PokyActivator.getDefault().setQEMUPassword(password);
												stdin.println(password);
												stdin.close();
											}
										} catch (Exception e) {
											UIUtils.handleVisualError("Unable to set password", e);
										}
									}
									else{
										process.destroy();
									}
								}

							});
						}
					}
				}
				catch(IOException e){

				}
			}
		};
		thread.start();
	}


}