package com.buglabs.dragonfly.cdt.poky.ui.wizard.pages;

import org.eclipse.cdt.ui.wizards.NewCProjectWizardPage;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

public class NewPokyProjectWizardPage extends NewCProjectWizardPage{

	private Text projectNameField;
	private Text kernelLocation;
	private Text rootfsLocation;
	private Text compilerRoot;
	private Text compilerPokyRoot;
	
	private GridData gridData;
	private GridData rbData;
	private GridData labelData;
	private GridData textData;
	private GridData gdButton;
	
	private Button rootBrowseBtn;
	private Button rootRadioBtn;
	private Button pokyRootRadioBtn;
	private Button pokyRootBrowseBtn;
	private String compiler;
	
	private static final String RADIO_BUTTON_TYPE = "RADIO_BUTTON_TYPE";
	private static final String RADIO_ROOT = "RADIO_ROOT";
	private static final String RADIO_POKY_ROOT = "RADIO_POKY_ROOT";

	public NewPokyProjectWizardPage(String pageName) {
		super(pageName);
	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));

		createProjectNameGroup(composite);

		setControl(composite);
	}
	
	protected final void createProjectNameGroup(Composite parent) {
		// project specification group
		Composite projectGroup = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		projectGroup.setLayout(layout);
		projectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// new project label
		Label projectLabel = new Label(projectGroup, SWT.NONE);
		projectLabel.setText("Project Name"); //$NON-NLS-1$
		projectLabel.setFont(parent.getFont());

		// new project name entry field
		projectNameField = new Text(projectGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		projectNameField.setLayoutData(data);
		projectNameField.setFont(parent.getFont());
		projectNameField.addListener(SWT.Modify, nameModifyListener);
		
		initGridData();

		Label compilerDescription = new Label(parent, SWT.NONE);
		compilerDescription
				.setText("Define toolchian that will be used in compilation process");

		Group compilerGroup = new Group(parent, SWT.NONE);
		compilerGroup.setText("Cross Compiler");
		compilerGroup.setLayout(new GridLayout(3, false));
		compilerGroup.setLayoutData(gridData);

		createCompilerOptionsSection(compilerGroup);

		Label targetDescription = new Label(parent, SWT.NONE);
		targetDescription.setText("Define kernel and root file system");

		Group targetGroup = new Group(parent, SWT.NONE);
		targetGroup.setText("Target Options");
		targetGroup.setLayout(new GridLayout(3, false));
		targetGroup.setLayoutData(gridData);

		createTargetOptionsSection(targetGroup);
	}
	
	private void initGridData() {
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.grabExcessHorizontalSpace = true;

		rbData = new GridData();
		rbData.widthHint = 300;
		rbData.horizontalSpan = 3;

		labelData = new GridData();
		labelData.horizontalIndent = 20;

		textData = new GridData(GridData.FILL_HORIZONTAL);
		textData.grabExcessHorizontalSpace = true;

		gdButton = new GridData();
		gdButton.horizontalAlignment = SWT.BEGINNING;
	}
	
	private void createCompilerOptionsSection(final Group compilerGroup) {

		Listener radioGroupListener = new Listener() {
			public void handleEvent(Event event) {
				if (event.widget instanceof Button) {
					Button b = (Button) event.widget;
					String data = (String) b.getData();
					if (data.equals(RADIO_ROOT)) {
						pokyRootBrowseBtn.setEnabled(false);
						compilerPokyRoot.setEnabled(false);

						rootBrowseBtn.setEnabled(true);
						compilerRoot.setEnabled(true);
					} else if (data.equals(RADIO_POKY_ROOT)) {
						rootBrowseBtn.setEnabled(false);
						compilerRoot.setEnabled(false);

						pokyRootBrowseBtn.setEnabled(true);
						compilerPokyRoot.setEnabled(true);
					}
				}
			}
		};

		rootRadioBtn = new Button(compilerGroup, SWT.RADIO);
		rootRadioBtn.setData(RADIO_ROOT);

		rootRadioBtn.setSelection(true);
		rootRadioBtn.setText("Use external toolchain");
		rootRadioBtn.setLayoutData(rbData);
		rootRadioBtn.addListener(SWT.Selection, radioGroupListener);

		Label rootLabel = new Label(compilerGroup, SWT.NONE);
		rootLabel.setText("Root ");
		rootLabel.setLayoutData(labelData);

		compilerRoot = new Text(compilerGroup, SWT.BORDER);
		compilerRoot.setLayoutData(textData);

		rootBrowseBtn = new Button(compilerGroup, SWT.PUSH);
		rootBrowseBtn.setText("Browse...");
		rootBrowseBtn.setLayoutData(gdButton);

		rootBrowseBtn.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseUp(MouseEvent e) {
				DirectoryDialog fd = new DirectoryDialog(((Button) e.widget)
						.getShell());
				compilerRoot.setText(fd.open());
				compiler = compilerRoot.getText();
			}

		});

		pokyRootRadioBtn = new Button(compilerGroup, SWT.RADIO);
		pokyRootRadioBtn.setSelection(false);
		pokyRootRadioBtn.setText("Use a full Poky tree");
		pokyRootRadioBtn.setLayoutData(rbData);
		pokyRootRadioBtn.setData(RADIO_POKY_ROOT);
		pokyRootRadioBtn.addListener(SWT.Selection, radioGroupListener);

		Label pokyRootLabel = new Label(compilerGroup, SWT.NONE);
		pokyRootLabel.setText("Poky Root ");
		pokyRootLabel.setLayoutData(labelData);

		compilerPokyRoot = new Text(compilerGroup, SWT.BORDER);
		compilerPokyRoot.setLayoutData(textData);

		pokyRootBrowseBtn = new Button(compilerGroup, SWT.PUSH);
		pokyRootBrowseBtn.setText("Browse...");
		pokyRootBrowseBtn.setLayoutData(gdButton);
		pokyRootBrowseBtn.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseUp(MouseEvent e) {
				DirectoryDialog fd = new DirectoryDialog(((Button) e.widget)
						.getShell());
				compilerPokyRoot.setText(fd.open());
				compiler = compilerPokyRoot.getText();
			}

		});

	}

	/*
	 * Creates target options group
	 */
	private void createTargetOptionsSection(Group targetGroup) {
		Label kernelLabel = new Label(targetGroup, SWT.NONE);
		kernelLabel.setText("Kernel");
		kernelLabel.setLayoutData(labelData);

		kernelLocation = new Text(targetGroup, SWT.BORDER);
		kernelLocation.setLayoutData(textData);
		/*kernelLocation.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent e) {
			}
		});*/

		Button kernelBtn = new Button(targetGroup, SWT.PUSH);
		kernelBtn.setText("Browse...");
		kernelBtn.setLayoutData(gdButton);

		kernelBtn.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseUp(MouseEvent e) {
				FileDialog fd = new FileDialog(((Button) e.widget).getShell());
				kernelLocation.setText(fd.open());
			}

		});

		Label rootfsLabel = new Label(targetGroup, SWT.NONE);
		rootfsLabel.setText("Root file system");
		rootfsLabel.setLayoutData(labelData);

		rootfsLocation = new Text(targetGroup, SWT.BORDER);
		rootfsLocation.setLayoutData(textData);

		Button rootfBtn = new Button(targetGroup, SWT.PUSH);
		rootfBtn.setText("Browse...");
		rootfBtn.setLayoutData(gdButton);

		rootfBtn.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseUp(MouseEvent e) {
				FileDialog fd = new FileDialog(((Button) e.widget).getShell());
				rootfsLocation.setText(fd.open());
			}

		});
	}
	
	// Listeners
	private Listener nameModifyListener = new Listener() {
		public void handleEvent(Event e) {
			boolean valid = validatePage();
			setPageComplete(valid);
			if(valid) {
			//	mysetLocationForSelection();
			//	mysetPackageNameForSelection();
			}
		}
	};


	public void setVisible(boolean visible) {
		//super.setVisible(visible);
		this.getControl().setVisible(visible);
		if (visible)
			projectNameField.setFocus();
	}

	protected boolean validatePage() {
		return true;
	}
	
	public boolean isPageComplete() {

		setErrorMessage(null);

		String projName = projectNameField.getText();

		char[] cs = projName.toCharArray();
		if (cs.length != 0) {
			for (int i = 0; i < cs.length; i++) {
				if (cs[i] == ' ' || cs[i] == '_') {
					continue;
				}
				if (!Character.isLetterOrDigit(cs[i])) {
					setErrorMessage("Invalid project name.");
					return false;
				}
			}
			
			if (ResourcesPlugin.getWorkspace().validateName(projName, IFile.PROJECT).getSeverity() != IStatus.OK) {
				setErrorMessage("Invalid project name.");
				return false;
			}

			IProject proj = ResourcesPlugin.getWorkspace().getRoot().getProject(projName);
			if (proj.exists()) {
				setErrorMessage("Project with name " + projName + " already exists.");
				return false;
			}
			return true;
		}
		return false;
	}

	public String getProjectName() {
		return projectNameField.getText();
	}
}
