package com.buglabs.dragonfly.cdt.poky.ui.wizard.pages;

import java.io.File;
import java.util.prefs.Preferences;

import org.eclipse.cdt.ui.wizards.NewCProjectWizardPage;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;;

public class NewSbProjectWizardPage extends NewCProjectWizardPage {//extends  WizardPage

	boolean useDefaults = true;

	boolean usePackageNameDefault = true;

	// initial value stores
	private String initialProjectFieldValue;
	private String initialLocationFieldValue;
	private String initialPackageNameFieldValue;

	// the value the user has entered
	String customLocationFieldValue;
	String customPackageNameFieldValue;

	// widgets
	Text projectNameField;
	Text locationPathField;
	Text packageNameField;
	Label locationLabel;
	Label packageNameLabel;
	Button browseButton;

	// Default workspace directory inside the Scratchbox environment
	IPath sbWsPath;
	// Scratchbox installation directory
	IPath sbRootPath;

	private Listener nameModifyListener = new Listener() {
		public void handleEvent(Event e) {
			boolean valid = validatePage();
			setPageComplete(valid);
			if(valid) {
				mysetLocationForSelection();
				mysetPackageNameForSelection();
			}
		}
	};

	public Listener locationModifyListener = new Listener() {
		public void handleEvent(Event e) {
			setPageComplete(validatePage());
		}
	};

	public Listener packageNameModifyListener = new Listener() {
		public void handleEvent(Event e) {
			setPageComplete(validatePage());
		}
	};

	// constants
	private static final int SIZING_TEXT_FIELD_WIDTH = 250;

	/**
	 * Creates a new project creation wizard page.
	 *
	 * @param pageName the name of this page
	 */
	public NewSbProjectWizardPage(String pageName) {//WizardNewProjectCreationPage(String pageName) {
		super(pageName);
		setPageComplete(false);
		customLocationFieldValue = ""; //$NON-NLS-1$
		customPackageNameFieldValue = ""; //$NON-NLS-1$

		// Form a default value for the workspace directory.
		String username = System.getProperty("user.name");
		sbWsPath = new Path("/home/" + username + "/workspace/");

		// Store the root path for validation of the project location.
		sbRootPath = new Path(sbWsPath.toOSString());

	}

	/** (non-Javadoc)
	 * Method declared on IDialogPage.
	 */
	public void createControl(Composite parent) {

		Composite composite = new Composite(parent, SWT.NULL);
		composite.setFont(parent.getFont());

		initializeDialogUnits(parent);

        PlatformUI.getWorkbench().getHelpSystem().setHelp(composite,
                "");

		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));

		createProjectNameGroup(composite);
		createProjectLocationGroup(composite);
		createPackageNameGroup(composite);
		setPageComplete(validatePage());
		// Show description on opening
		setErrorMessage(null);
		setMessage(null);
		setControl(composite);
	}

	/**
	 * Creates the project location specification controls.
	 *
	 * @param parent the parent composite
	 */
	protected final void createProjectLocationGroup(Composite parent) {
		createProjectLocationGroup(parent, 3);
	}

	protected final void createProjectLocationGroup(Composite parent, int iSpan) {

		Font font = parent.getFont();
		// project specification group
		Group projectGroup = new Group(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = iSpan;
		projectGroup.setLayout(layout);
		projectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		projectGroup.setFont(font);
		projectGroup.setText("Group"); //$NON-NLS-1$

		createProjectLocationGroupUseDefaultsButton(projectGroup,iSpan);
		createUserSpecifiedProjectLocationGroup(projectGroup, !useDefaults);

	}

	protected final void createProjectLocationGroupUseDefaultsButton(Composite parent) {
		createProjectLocationGroupUseDefaultsButton(parent, 3);
	}

	protected final void createProjectLocationGroupUseDefaultsButton(Composite parent, int iSpan) {
		Font font = parent.getFont();

		final Button useDefaultsButton =
			new Button(parent, SWT.CHECK | SWT.RIGHT);
		useDefaultsButton.setText("SSS");
		useDefaultsButton.setSelection(useDefaults);
		useDefaultsButton.setFont(font);

		GridData buttonData = new GridData();
		buttonData.horizontalSpan = iSpan;
		useDefaultsButton.setLayoutData(buttonData);

		SelectionListener listener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				useDefaults = useDefaultsButton.getSelection();
				browseButton.setEnabled(!useDefaults);
				locationPathField.setEnabled(!useDefaults);
				locationLabel.setEnabled(!useDefaults);
				if (useDefaults) {
					customLocationFieldValue = locationPathField.getText();
					mysetLocationForSelection();
				} else {
					if (customLocationFieldValue == "") {
						// If pressed first time to use user settings, prefill
						// the value from default to make dialog more user friendly
						customLocationFieldValue = locationPathField.getText();
					}
					locationPathField.setText(customLocationFieldValue);
				}
			}
		};
		useDefaultsButton.addSelectionListener(listener);

	}


	/**
	 * Creates the project name specification controls.
	 *
	 * @param parent the parent composite
	 */
	protected final void createProjectNameGroup(Composite parent) {
		// project specification group
		Composite projectGroup = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		projectGroup.setLayout(layout);
		projectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// new project label
		Label projectLabel = new Label(projectGroup, SWT.NONE);
		projectLabel.setText("ddd"); //$NON-NLS-1$
		projectLabel.setFont(parent.getFont());

		// new project name entry field
		projectNameField = new Text(projectGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = SIZING_TEXT_FIELD_WIDTH;
		projectNameField.setLayoutData(data);
		projectNameField.setFont(parent.getFont());

		// Set the initial value first before listener
		// to avoid handling an event during the creation.
		if (initialProjectFieldValue != null)
			projectNameField.setText(initialProjectFieldValue);
		projectNameField.addListener(SWT.Modify, nameModifyListener);
	}

	/**
	 * Creates the project location specification controls.
	 *
	 * @param projectGroup the parent composite
	 * @param boolean - the initial enabled state of the widgets created
	 */
	protected void createUserSpecifiedProjectLocationGroup(
			Composite projectGroup,
			boolean enabled) {
		int cols=0;

		try {
			GridLayout lo = (GridLayout)(projectGroup.getLayout());
			cols=lo.numColumns;
		} catch (Exception e) {
			cols=3;
		}

		createUserSpecifiedProjectLocationGroup(projectGroup,enabled, cols);

	}

	protected void createUserSpecifiedProjectLocationGroup(
			Composite projectGroup,
			boolean enabled, int iSpan) {

		Font font = projectGroup.getFont();
		GridLayout oLayout = new GridLayout();

		oLayout.numColumns = iSpan;
		projectGroup.setLayout(oLayout);

		// location label
		locationLabel = new Label(projectGroup, SWT.NONE);
		locationLabel.setText("sass"); //$NON-NLS-1$
		locationLabel.setEnabled(enabled);
		locationLabel.setFont(font);

		// project location entry field
		locationPathField = new Text(projectGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = SIZING_TEXT_FIELD_WIDTH;
		locationPathField.setLayoutData(data);
		locationPathField.setEnabled(enabled);
		locationPathField.setFont(font);

		// browse button
		browseButton = new Button(projectGroup, SWT.PUSH);
		browseButton.setText("browse"); //$NON-NLS-1$
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				myhandleLocationBrowseButtonPressed();
			}
		});

		browseButton.setEnabled(enabled);
		browseButton.setFont(font);
		setButtonLayoutData(browseButton);

		// Set the initial value first before listener
		// to avoid handling an event during the creation.
		if (initialLocationFieldValue == null)
			//locationPathField.setText(Platform.getLocation().toOSString());
			locationPathField.setText(sbWsPath.toOSString());
		else
			locationPathField.setText(initialLocationFieldValue);
		locationPathField.addListener(SWT.Modify, locationModifyListener);
	}

	/**
	 * Creates the package name specification controls.
	 *
	 * @param parent the parent composite
	 */
	protected void createPackageNameGroup(Composite parent) {

		Font font = parent.getFont();
		// project specification group
		Group packageNameGroup = new Group(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		packageNameGroup.setLayout(layout);
		packageNameGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		packageNameGroup.setFont(font);
		packageNameGroup.setText("pacakge"); //$NON-NLS-1$

		final Button useDefaultsButton =
			new Button(packageNameGroup, SWT.CHECK | SWT.RIGHT);
		useDefaultsButton.setText("use defaults"); //$NON-NLS-1$
		useDefaultsButton.setSelection(usePackageNameDefault);
		useDefaultsButton.setFont(font);

		GridData buttonData = new GridData();
		buttonData.horizontalSpan = 2;
		useDefaultsButton.setLayoutData(buttonData);

		createUserSpecifiedPackageNameGroup(packageNameGroup, !usePackageNameDefault);

		SelectionListener listener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				usePackageNameDefault = useDefaultsButton.getSelection();
				packageNameField.setEnabled(!usePackageNameDefault);
				packageNameLabel.setEnabled(!usePackageNameDefault);
				if (usePackageNameDefault) {
					customPackageNameFieldValue = packageNameField.getText();
					mysetPackageNameForSelection();

				} else {
					packageNameField.setText(customPackageNameFieldValue);
				}
			}
		};
		useDefaultsButton.addSelectionListener(listener);
	}

	/**
	 * Creates the package name field controls.
	 *
	 * @param packageNameGroup the parent composite
	 * @param enabled - the initial enabled state of the widgets created
	 */
	protected void createUserSpecifiedPackageNameGroup(
		Composite packageNameGroup,
		boolean enabled) {

		Font font = packageNameGroup.getFont();

		// location label
		packageNameLabel = new Label(packageNameGroup, SWT.NONE);
		packageNameLabel.setText("asdasd"); //$NON-NLS-1$
		packageNameLabel.setEnabled(enabled);
		packageNameLabel.setFont(font);

		// project location entry field
		packageNameField = new Text(packageNameGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		packageNameField.setLayoutData(data);
		packageNameField.setEnabled(enabled);
		packageNameField.setFont(font);

		// Set the initial value first before listener
		// to avoid handling an event during the creation.
		if (initialPackageNameFieldValue == null){
			//locationPathField.setText(Platform.getLocation().toOSString());
			packageNameField.setText(""); //$NON-NLS-1$
		}
		else {
			packageNameField.setText(initialPackageNameFieldValue);
		}

		packageNameField.addListener(SWT.Modify, packageNameModifyListener);
	}

	/**
	 * Returns the current project location path as entered by
	 * the user, or its anticipated initial value.
	 * Note that if the default has been returned the path
	 * in a project description used to create a project
	 * should not be set.
	 *
	 * @return the project location path or its anticipated initial value.
	 */
	public IPath getLocationPath() {
		if (useDefaults) {
//			TODO: why did this return the workspace root originally?
//			return Platform.getLocation();
			IPath projectPath = sbWsPath.append(getProjectName());
			return (projectPath);
		}

		return new Path(getProjectLocationFieldValue());
	}
	/**
	 * Creates a project resource handle for the current project name field value.
	 * <p>
	 * This method does not create the project resource; this is the responsibility
	 * of <code>IProject::create</code> invoked by the new project resource wizard.
	 * </p>
	 *
	 * @return the new project resource handle
	 */
	public IProject getProjectHandle() {
	 return	ResourcesPlugin.getWorkspace().getRoot().getProject(
                getProjectName());
	 	/* There can be only one root open at given time, and to make
	 	 * 'templateWorkspace class is quite task, but it would be nice */
	}

	/**
	 * Returns the current project name as entered by the user, or its anticipated
	 * initial value.
	 *
	 * @return the project name, its anticipated initial value, or <code>null</code>
	 *   if no project name is known
	 */
	public String getProjectName() {
		if (projectNameField == null)
			return initialProjectFieldValue;

		return getProjectNameFieldValue();
	}

	/**
	 * Returns the value of the project name field
	 * with leading and trailing spaces removed.
	 *
	 * @return the project name in the field
	 */
	private String getProjectNameFieldValue() {
		if (projectNameField == null)
			return ""; //$NON-NLS-1$
		else
			return projectNameField.getText().trim();
	}
	/**
	 * Returns the value of the project location field
	 * with leading and trailing spaces removed.
	 *
	 * @return the project location directory in the field
	 */
	private String getProjectLocationFieldValue() {
		if (locationPathField == null)
			return ""; //$NON-NLS-1$
		else
			return locationPathField.getText().trim();
	}

	/**
	 *	Open an appropriate directory browser
	 */
	private void myhandleLocationBrowseButtonPressed() {
		//TODO figure out is the private needed
		DirectoryDialog dialog =
			new DirectoryDialog(locationPathField.getShell());
		dialog.setMessage("dir"); //$NON-NLS-1$

		String dirName = getProjectLocationFieldValue();
		if (!dirName.equals("")) { //$NON-NLS-1$
			File path = new File(dirName);
			if (path.exists())
				dialog.setFilterPath(new Path(dirName).toOSString());
		}

		String selectedDirectory = dialog.open();
		if (selectedDirectory != null) {
			customLocationFieldValue = selectedDirectory;
			locationPathField.setText(customLocationFieldValue);
		}
	}
	/**
	 * Sets the initial project name that this page will use when
	 * created. The name is ignored if the createControl(Composite)
	 * method has already been called. Leading and trailing spaces
	 * in the name are ignored.
	 *
	 * @param name initial project name for this page
	 */

	public void setInitialProjectName(String name) {
		if (name == null)
			initialProjectFieldValue = null;
		else {
			initialProjectFieldValue = name.trim();
			initialLocationFieldValue =
				getDefaultLocationForName(initialProjectFieldValue);
		}
	}

	/**
	 * Set the location to the default location if we are set to useDefaults.
	 */
	private void mysetLocationForSelection() {
		//TODO figure if the private is needed
		if (useDefaults)
			locationPathField.setText(
				getDefaultLocationForName(getProjectNameFieldValue()));
	}

	/**
	 * Set the location to the default location if we are set to
	 * usePackageNameDefault.
	 */
	private void mysetPackageNameForSelection() {
		if (usePackageNameDefault) {
			packageNameField.setText(
					getDefaultPackageNameForName(getProjectName()));
		}
	}

	private String getDefaultPackageNameForName(String nameValue) {
		return nameValue;
	}

	/**
	 * Get the defualt location for the provided name.
	 * @return String
	 * @param String
	 */
	private String getDefaultLocationForName(String nameValue) {
		//IPath defaultPath = Platform.getLocation().append(nameValue);
		IPath defaultPath = sbWsPath.append(nameValue);
		return defaultPath.toOSString();
	}

	/**
	 * Returns whether this page's controls currently all contain valid
	 * values.
	 *
	 * @return <code>true</code> if all controls are valid, and
	 *   <code>false</code> if at least one is invalid
	 */
	protected boolean validatePage() {
		return true;
	}

	/*
	 * see @DialogPage.setVisible(boolean)
	 * Modified from original M.Sillanpaas code, Createcontens does not call
	 * Super method so this has to call Super either
	 */
	public void setVisible(boolean visible) {
		//super.setVisible(visible);
		this.getControl().setVisible(visible);
		if (visible)
			projectNameField.setFocus();
	}

	/**
	 * Returns the useDefaults.
	 * @return boolean
	 */
	public boolean useDefaults() {
		return useDefaults;
	}

	public boolean getPackageNameUseDefaults() {
		return usePackageNameDefault;
	}

	public boolean setPackageNameUseDefaults(boolean setTo) {
		return usePackageNameDefault=setTo;
	}

	public String getPackageName() {
		String sText = packageNameField.getText();
		if(usePackageNameDefault ) {
			if ( sText.length() == 0) {
				return getProjectName();
			} else {
				return sText;
			}
		} else {
			return sText;
		}
	}

}

